unit uTwmPop3Client;

{$i+ raise exceptions on I/O errors}
{$h+}

interface

uses
{$ifdef os2}
  OS2Def,
  Os2Base,
{$else}
  Windows,
  Os2Comp,
{$endif}
  SysUtils,
  uTwmSocket,
  uTwmSockDef,
  uTwmSockClient;

type
  ePop3=class(eSockClient);

type // abstract class for retrieving a Pop3Message
  tPop3Message=class
    procedure AddLine(_Line: AnsiString); virtual; abstract;
  end;

type
  tPop3Client=class(tSockClient)
  protected
    fUsername: string;
    fPassword: string;
    procedure SignOn; override;
    procedure SignOff; override;
    function GetMsgsAvail: LongInt; virtual;
  public
    constructor Create;
    constructor CreateEx(_Hostname, _Username, _Password: string);
    function CheckMail(var _MsgCount, _MsgSize: LongInt): boolean; virtual;
    procedure GetMsg(_MsgNo: LongInt; _Msg: tPop3Message); virtual;
    procedure GetHeader(_MsgNo: LongInt; _Msg: tPop3Message); virtual;
    procedure DeleteMsg(_MsgNo: integer); virtual;
    procedure GetMsgUids(_UidList: tPop3Message); virtual;
    property Username: string read fUsername write fUsername;
    property Password: string read fPassword write fPassword;
    property MsgsAvail: LongInt read GetMsgsAvail;
  end;

implementation

uses
  uTwmStringS;

const
  cService='pop3';

constructor tPop3Client.Create;
  begin
  inherited Create;
  fService:=cService;
  fPassword:='';
end;

constructor tPop3Client.CreateEx(_Hostname, _Username, _Password: string);
  begin
  Create;
  Hostname:=_Hostname;
  Username:=_Username;
  Password:=_Password;
end;

procedure tPop3Client.SignOn;
  var
    s: AnsiString;
  begin
  if fUsername='' then
    raise ePop3.Create('no username specified');
  if fPassword='' then
    raise ePop3.Create('no password specified');
  s:='USER '+fUsername;
  LogLine('> '+s);
  fSocket.SendStrLn(s);
  while not fSocket.DataAvailable do
    DosSleep(10);
  fSocket.RecvStrLn(s);
    LogLine('< '+s);
  if LeftStr(s, 3)<>'+OK' then
    raise ePop3.Create(TailStr(s, 6));
  s:='PASS ';
  LogLine('> '+s+'****************');
  fSocket.SendStrLn(s+fPassword);
  while not fSocket.DataAvailable do
    DosSleep(10);
  fSocket.RecvStrLn(s);
  LogLine('< '+s);
  if LeftStr(s, 3)<>'+OK' then
    Raise ePop3.Create(TailStr(s, 6));
end;

procedure tPop3Client.SignOff;
  var
    s: AnsiString;
  begin
  s:='QUIT';
  LogLine('> '+s);
  fSocket.SendStrLn(s);
  fSocket.RecvStrLn(s);
  LogLine('< '+s);
end;

function tPop3Client.CheckMail(var _MsgCount, _MsgSize: LongInt): boolean;
  var
    s: AnsiString;
  begin
  if not Connected then
    raise ePop3.Create('not in transaction state');
  s:='STAT';
  LogLine('> '+s);
  fSocket.SendStrLn(s);
  while not fSocket.DataAvailable do
    DosSleep(10);
  fSocket.RecvStrLn(s);
  LogLine('< '+s);
  if LeftStr(s, 3)<>'+OK' then
    raise ePop3.Create(TailStr(s, 4));
  _MsgCount:=Dec2Long(NthWord(s, 2, ' '));
  _MsgSize:=Dec2Long(NthWord(s, 3, ' '));
  Result:=_MsgCount<>0;
end;

function tPop3Client.GetMsgsAvail: LongInt;
  var
    MsgSize: LongInt;
  begin
  CheckMail(Result, MsgSize);
end;

procedure tPop3Client.GetMsg(_MsgNo: LongInt; _Msg: tPop3Message);
  var
    s: AnsiString;
    eClass: ExceptClass;
    eMessage: string;
    eHelp: integer;
    eAddr: pointer;
  begin
  eClass:=nil;
  try
    s:='RETR '+Long2Dec(_MsgNo);
    LogLine('> '+s);
    fSocket.SendStrLn(s);
    while not fSocket.DataAvailable do
      DosSleep(5);
    fSocket.RecvStrLn(s);
    LogLine('< '+s);
    repeat
      repeat
      until fSocket.RecvStrLn(s);
      if s='.' then begin
        LogLine('< '+s);
        Break;
      end;
      if not Assigned(eClass) then
        try // make sure an exception in AddLine does not trash the connection
          _Msg.AddLine(s);
        except
          on e: exception do begin
            eClass:=ExceptClass(e.ClassType);
            eMessage:=e.Message;
            eHelp:=e.HelpContext;
            eAddr:=ExceptAddr;
          end;
        end;
    until false;
  finally
    if Assigned(eClass) then
      raise eClass.CreateHelp(eMessage, eHelp) at eAddr;
  end;
end;

procedure tPop3Client.GetHeader(_MsgNo: LongInt; _Msg: tPop3Message);
  var
    s: AnsiString;
    eClass: ExceptClass;
    eMessage: string;
    eHelp: integer;
    eAddr: pointer;
  begin
  eClass:=nil;
  try
    s:='TOP '+Long2Dec(_MsgNo)+' 0';
    LogLine('> '+s);
    fSocket.SendStrLn(s);
    while not fSocket.DataAvailable do
      DosSleep(5);
    fSocket.RecvStrLn(s);
    LogLine('< '+s);
    repeat
      repeat
      until fSocket.RecvStrLn(s);
      if s='.' then begin
        LogLine('< '+s);
        Break;
      end;
      if not Assigned(eClass) then
        try // make sure an exception in AddLine does not trash the connection
          _Msg.AddLine(s);
        except
          on e: exception do begin
            eClass:=ExceptClass(e.ClassType);
            eMessage:=e.Message;
            eHelp:=e.HelpContext;
            eAddr:=ExceptAddr;
          end;
        end;
    until false;
  finally
    if Assigned(eClass) then
      raise eClass.CreateHelp(eMessage, eHelp) at eAddr;
  end;
end;

procedure tPop3Client.GetMsgUids(_UidList: tPop3Message);
  var
    s: AnsiString;
    eClass: ExceptClass;
    eMessage: string;
    eHelp: integer;
    eAddr: pointer;
  begin
  eClass:=nil;
  try
    s:='UIDL';
    LogLine('> '+s);
    fSocket.SendStrLn(s);
    while not fSocket.DataAvailable do
      DosSleep(5);
    fSocket.RecvStrLn(s);
    LogLine('< '+s);
    repeat
      repeat
      until fSocket.RecvStrLn(s);
      if s='.' then begin
        LogLine('< '+s);
        Break;
      end;
      if not Assigned(eClass) then
        try // make sure an exception in AddLine does not trash the connection
          _UidList.AddLine(s);
        except
          on e: exception do begin
            eClass:=ExceptClass(e.ClassType);
            eMessage:=e.Message;
            eHelp:=e.HelpContext;
            eAddr:=ExceptAddr;
          end;
        end;
    until false;
  finally
    if Assigned(eClass) then
      raise eClass.CreateHelp(eMessage, eHelp) at eAddr;
  end;
end;

procedure tPop3Client.DeleteMsg(_MsgNo: integer);
  var
    s: AnsiString;
  begin
  s:='DELE '+Long2Dec(_MsgNo);
  LogLine('> '+s);
  fSocket.SendStrLn(s);
  while not fSocket.DataAvailable do
    DosSleep(10);
  fSocket.RecvStrLn(s);
    LogLine('< '+s);
  if LeftStr(s, 3)<>'+OK' then
    raise ePop3.Create(TailStr(s, 6));
end;

end.
