unit uTwmSmtpClient;

interface

uses
{$ifdef os2}
  OS2Def,
  Os2Base,
{$else}
  Windows,
  Os2Comp,
{$endif}
  SysUtils,
  uTwmSocket,
  uTwmSockDef,
  uTwmSockClient;

type
  eSmtp=class(eSockClient);

type
  eSendMessage=class(eSmtp);

type // abstract class for sending a SmtpMessage
  tSmtpMessage=class
    procedure Reset; virtual; abstract;
    function EOM: boolean; virtual; abstract;
    function GetNextLine: ShortString; virtual; abstract;
  end;

type
  tSmtpClient=class(tSockClient)
  protected
    procedure SignOn; override;
    procedure SignOff; override;
    procedure WriteAndCheck(_s: AnsiString; const _Expected: ShortString); virtual;
    function AddressOnly(var _Line: ShortString): ShortString; virtual;
  public
    constructor Create;
    constructor CreateEx(_Hostname: ShortString);
    procedure SendMessage(_Msg: tSmtpMessage); virtual;
  end;

implementation

uses
  uTwmStringS;

const
  cService='smtp';

constructor tSmtpClient.Create;
  begin
  inherited Create;
  fService:=cService;
end;

constructor tSmtpClient.CreateEx(_Hostname: ShortString);
  begin
  Create;
  fHostname:=_Hostname;
end;

procedure tSmtpClient.WriteAndCheck(_s: AnsiString; const _Expected: ShortString);
  begin
  WriteLn('> ', _s);
  fSocket.SendStrLn(_s);
  while not fSocket.DataAvailable do
    DosSleep(10);
  fSocket.RecvStrLn(_s);
    WriteLn('< ', _s);
  if LeftStr(_s, 3)<>_Expected then
    raise eSmtp.Create(TailStr(_s, 5));
end;

procedure tSmtpClient.SignOn;
  begin
  WriteAndCheck('HELO '+fSocket.GetLocalHost, '250');
end;

procedure tSmtpClient.SignOff;
begin
  WriteAndCheck('QUIT', '221');
end;

function tSmtpClient.AddressOnly(var _Line: ShortString): ShortString;
var
  p: integer;
begin
  p:=Pos('<', _Line);
  if p=0 then
    raise eSendMessage.Create('illegal address: '+_Line);
  Result:=TailStr(_Line, p);
  p:=Pos('>', Result);
  if p=0 then
    raise eSendMessage.Create('illegal address: '+_Line);
  _Line:=TailStr(Result, p+1);
  Result:=LeftStr(Result, p);
end;

function ExtractFirstAddress(var _Line: ShortString): ShortString;
  var
    p: integer;
  begin
  p:=Pos(',', _Line);
  if p=0 then begin
    Result:=_Line;
    _Line:='';
  end else begin
    Result:=Trim(LeftStr(_Line, p-1));
    _Line:=Trim(TailStr(_Line, p+1));
  end;
end;

procedure tSmtpClient.SendMessage(_Msg: tSmtpMessage);

  function UnfoldField(var _Start: ShortString): ShortString;
    begin
    Result:=_Start;
    _Start:=_Msg.GetNextLine;
    while (_Start<>'') and (_Start[1] in [' ', #9]) do begin
      Result:=Result+' '+Trim(_Start);
      _Start:=_Msg.GetNextLine;
    end;
  end;

  var
    Found: boolean;
    ToAddr: ShortString;
    FromAddr: ShortString;
    Line: ShortString;
  begin
  Line:=_Msg.GetNextLine;
  if UpperCase(LeftStr(Line, 6))<>'FROM: ' then
    raise eSendMessage.Create('No sender in message');
  Line:=TailStr(Line, 7);
  FromAddr:=UnfoldField(Line);

  WriteAndCheck('MAIL FROM:'+AddressOnly(FromAddr), '250');
  try
    if UpperCase(LeftStr(Line, 4))<>'TO: ' then
      raise eSendMessage.Create('No recipient in message');
    Line:=TailStr(Line, 5);
    ToAddr:=UnfoldField(Line);
    Found:=true;
    while Found do begin
      while ToAddr<>'' do
//        WriteAndCheck('RCPT TO:'+AddressOnly(ToAddr), '250');
        WriteAndCheck('RCPT TO:'+ExtractFirstAddress(ToAddr), '250');
      if UpperCase(LeftStr(Line, 4))='TO: ' then begin
        Line:=TailStr(Line, 5);
        ToAddr:=UnfoldField(Line);
      end else if UpperCase(LeftStr(Line, 4))='CC: ' then begin
        Line:=TailStr(Line, 5);
        ToAddr:=UnfoldField(Line);
      end else if UpperCase(LeftStr(Line, 5))='BCC: ' then begin
        Line:=TailStr(Line, 6);
        ToAddr:=UnfoldField(Line);
      end else
        Found:=False;
    end;
    WriteAndCheck('DATA', '354');
    _Msg.Reset;
    Line:=_Msg.GetNextLine;
    UnfoldField(Line); // do not send sender line
    fSocket.SendStrLn(Line);
    while not _Msg.EOM do
      fSocket.SendStrLn(_Msg.GetNextLine);
    WriteAndCheck('.', '250');
  except
    WriteAndCheck('RSET', '250');
    raise;
  end;
end;


end.
